@@ -156,7 +156,7 @@ static double syscall_timeout[] = {
 static bool syscall_valid(const int syscall)
 {
 	return (syscall >= 0) &&
-	       (syscall <= (int)syscalls_len);
+	       (syscall < (int)syscalls_len);
 }
 
 #ifdef SYS_connect
@@ -1269,25 +1269,33 @@ static void syscall_account_sync_file(syscall_sync_info_t *info, const int sysca
 {
 	syscall_sync_file_t *f;
 	fd_cache_t *fc;
+	char *filename;
 	link_t *l;
 
 	if ((fc = syscall_fd_cache_lookup(pid, fd)) == NULL)
 		return;
 
-	for (l = info->sync_file.head; l; l = l->next) {
-		f = (syscall_sync_file_t *)l->data;
-		if ((f->syscall == syscall) && !strcmp(f->filename, fc->filename)) {
-			f->count++;
-			return;
+	pthread_mutex_lock(&fc->mutex);
+	filename = fc->filename;
+	if (filename) {
+		for (l = info->sync_file.head; l; l = l->next) {
+			f = (syscall_sync_file_t *)l->data;
+			if ((f->syscall == syscall) && !strcmp(f->filename, filename)) {
+				f->count++;
+				pthread_mutex_unlock(&fc->mutex);
+				return;
+			}
 		}
 	}
 
 	if ((f = calloc(1, sizeof(*f))) == NULL) {
 		health_check_out_of_memory("allocating file sync filename info");
+		pthread_mutex_unlock(&fc->mutex);
 		return;
 	}
 
 	f->filename = strdup(fc->filename);
+	pthread_mutex_unlock(&fc->mutex);
 	f->syscall = syscall;
 	f->count = 1;
 
@@ -2060,7 +2068,7 @@ void syscall_dump_inotify(json_object *j_obj, double duration)
 }
 #endif
 
-#ifdef SYS_execve
+#if defined(SYS_execve) || defined(SYS_execveat)
 /*
  *  syscall_execve_args()
  *	trace filenames used by execve()
@@ -2072,10 +2080,22 @@ static void syscall_execve_args(
 {
 	unsigned long args[sc->arg + 1];
 	char *filename;
+	int idx;
+
+	switch (sc->syscall) {
+	case SYS_execve:
+		idx = 0;
+		break;
+	case SYS_execveat:
+		idx = 1;
+		break;
+	default:
+		return;
+	}
 
 	(void)s;
 	syscall_get_args(pid, sc->arg, args);
-	filename = syscall_peek_filename(pid, args[0]);
+	filename = syscall_peek_filename(pid, args[idx]);
 	if (filename) {
 		syscall_add_filename(sc->syscall, pid, filename);
 		free(filename);
@@ -2439,8 +2459,9 @@ static syscall_info_t *syscall_count_usage(
 
 	if (sc->call_enter)
 		sc->call_enter(sc, s, pid);
-	else if (sc->call_enter_timeout)
+	else if (sc->call_enter_timeout) {
 		sc->call_enter_timeout(sc, s, pid, *(sc->threshold), timeout);
+	}
 
 	return s;
 }
@@ -2922,6 +2943,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_bind
 	SYSCALL(bind),
 #endif
+#ifdef SYS_bpf
+	SYSCALL(bpf),
+#endif
 #ifdef SYS_break
 	SYSCALL(break),
 #endif
@@ -2967,12 +2991,21 @@ syscall_t syscalls[] = {
 #ifdef SYS_clone
 	SYSCALL(clone),
 #endif
+#ifdef SYS_clone2
+	SYSCALL(clone2),
+#endif
+#ifdef SYS_clone3
+	SYSCALL(clone3),
+#endif
 #ifdef SYS_close
 	SYSCALL_CHK(close, 0, syscall_close_args, NULL),
 #endif
 #ifdef SYS_connect
 	SYSCALL_CHK(connect, 0, syscall_connect_args, syscall_connect_ret),
 #endif
+#ifdef SYS_copy_file_range
+	SYSCALL(copy_file_range),
+#endif
 #ifdef SYS_creat
 	SYSCALL(creat),
 #endif
@@ -3021,6 +3054,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_execve
 	SYSCALL_CHK(execve, 0, syscall_execve_args, NULL),
 #endif
+#ifdef SYS_execveat
+	SYSCALL_CHK(execveat, 0, syscall_execve_args, NULL),
+#endif
 #ifdef SYS_exit
 	SYSCALL_CHK(exit, 0, syscall_exit_args, NULL),
 #endif
@@ -3093,6 +3129,15 @@ syscall_t syscalls[] = {
 #ifdef SYS_fsetxattr
 	SYSCALL(fsetxattr),
 #endif
+#ifdef SYS_fsmount
+	SYSCALL(fsmount),
+#endif
+#ifdef SYS_fsopen
+	SYSCALL(fsopen),
+#endif
+#ifdef SYS_fspick
+	SYSCALL(fspick),
+#endif
 #ifdef SYS_fstat
 	SYSCALL(fstat),
 #endif
@@ -3192,6 +3237,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_getpriority
 	SYSCALL(getpriority),
 #endif
+#ifdef SYS_getrandom
+	SYSCALL(getrandom),
+#endif
 #ifdef SYS_getresgid
 	SYSCALL(getresgid),
 #endif
@@ -3273,6 +3321,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_io_getevents
 	SYSCALL(io_getevents),
 #endif
+#ifdef SYS_io_pgetevents
+	SYSCALL(io_pgetevents),
+#endif
 #ifdef SYS_ioperm
 	SYSCALL(ioperm),
 #endif
@@ -3291,12 +3342,24 @@ syscall_t syscalls[] = {
 #ifdef SYS_io_submit
 	SYSCALL(io_submit),
 #endif
+#ifdef SYS_io_uring_enter
+	SYSCALL(io_uring_enter),
+#endif
+#ifdef SYS_io_uring_register
+	SYSCALL(io_uring_register),
+#endif
+#ifdef SYS_io_uring_setup_
+	SYSCALL(io_uring_setup),
+#endif
 #ifdef SYS_ipc
 	SYSCALL(ipc),
 #endif
 #ifdef SYS_kcmp
 	SYSCALL(kcmp),
 #endif
+#ifdef SYS_kexec_file_load
+	SYSCALL(kexec_file_load),
+#endif
 #ifdef SYS_kexec_load
 	SYSCALL(kexec_load),
 #endif
@@ -3357,9 +3420,21 @@ syscall_t syscalls[] = {
 #ifdef SYS_madvise
 	SYSCALL(madvise),
 #endif
+#ifdef SYS_map_shadow_stack
+	SYSCALL(map_shadow_stack),
+#endif
 #ifdef SYS_mbind
 	SYSCALL(mbind),
 #endif
+#ifdef SYS_memory_ordering
+	SYSCALL(memory_ordering),
+#endif
+#ifdef SYS_membarrier
+	SYSCALL(membarrier),
+#endif
+#ifdef SYS_memfd_create
+	SYSCALL(memfd_create),
+#endif
 #ifdef SYS_migrate_pages
 	SYSCALL(migrate_pages),
 #endif
@@ -3381,6 +3456,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_mlock
 	SYSCALL(mlock),
 #endif
+#ifdef SYS_mlock2
+	SYSCALL(mlock2),
+#endif
 #ifdef SYS_mlockall
 	SYSCALL(mlockall),
 #endif
@@ -3396,6 +3474,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_mount
 	SYSCALL(mount),
 #endif
+#ifdef SYS_move_mount
+	SYSCALL(move_mount),
+#endif
 #ifdef SYS_move_pages
 	SYSCALL(move_pages),
 #endif
@@ -3492,6 +3573,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_open_by_handle_at
 	SYSCALL(open_by_handle_at),
 #endif
+#ifdef SYS_open_tree
+	SYSCALL(open_tree),
+#endif
 #ifdef SYS_pause
 	SYSCALL(pause),
 #endif
@@ -3501,6 +3585,12 @@ syscall_t syscalls[] = {
 #ifdef SYS_personality
 	SYSCALL(personality),
 #endif
+#ifdef SYS_pidfd_open
+	SYSCALL(pidfd_open),
+#endif
+#ifdef SYS_pidfd_send_signal
+	SYSCALL(pidfd_send_signal),
+#endif
 #ifdef SYS_pipe
 	SYSCALL(pipe),
 #endif
@@ -3510,6 +3600,21 @@ syscall_t syscalls[] = {
 #ifdef SYS_pivot_root
 	SYSCALL(pivot_root),
 #endif
+#ifdef SYS_pkey_alloc
+	SYSCALL(pkey_alloc),
+#endif
+#ifdef SYS_pkey_free
+	SYSCALL(pkey_free),
+#endif
+#ifdef SYS_pkey_get
+	SYSCALL(pkey_get),
+#endif
+#ifdef SYS_pkey_mprotect
+	SYSCALL(pkey_mprotect),
+#endif
+#ifdef SYS_pkey_set
+	SYSCALL(pkey_set),
+#endif
 #ifdef SYS_poll
 	SYSCALL_CHK_TIMEOUT(poll, 2, syscall_timeout_millisec, syscall_poll_generic_ret),
 #endif
@@ -3525,6 +3630,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_preadv
 	SYSCALL(preadv),
 #endif
+#ifdef SYS_preadv2
+	SYSCALL(preadv2),
+#endif
 #ifdef SYS_prlimit64
 	SYSCALL(prlimit64),
 #endif
@@ -3555,12 +3663,18 @@ syscall_t syscalls[] = {
 #ifdef SYS_pwritev
 	SYSCALL(pwritev),
 #endif
+#ifdef SYS_pwritev2
+	SYSCALL(pwritev2),
+#endif
 #ifdef SYS_query_module
 	SYSCALL(query_module),
 #endif
 #ifdef SYS_quotactl
 	SYSCALL(quotactl),
 #endif
+#ifdef SYS_quotactl_fd
+	SYSCALL(quotactl_fd),
+#endif
 #ifdef SYS_read
 	SYSCALL(read),
 #endif
@@ -3603,6 +3717,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_renameat
 	SYSCALL(renameat),
 #endif
+#ifdef SYS_renameat2
+	SYSCALL(renameat2),
+#endif
 #ifdef SYS_request_key
 	SYSCALL(request_key),
 #endif
@@ -3612,6 +3729,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_rmdir
 	SYSCALL(rmdir),
 #endif
+#ifdef SYS_rseq
+	SYSCALL(rseq),
+#endif
 #ifdef SYS_rt_sigaction
 	SYSCALL(rt_sigaction),
 #endif
@@ -3639,6 +3759,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_sched_getaffinity
 	SYSCALL(sched_getaffinity),
 #endif
+#ifdef SYS_sched_getattr
+	SYSCALL(sched_getattr),
+#endif
 #ifdef SYS_sched_getparam
 	SYSCALL(sched_getparam),
 #endif
@@ -3657,6 +3780,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_sched_setaffinity
 	SYSCALL(sched_setaffinity),
 #endif
+#ifdef SYS_sched_setattr
+	SYSCALL(sched_setattr),
+#endif
 #ifdef SYS_sched_setparam
 	SYSCALL(sched_setparam),
 #endif
@@ -3666,6 +3792,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_sched_yield
 	SYSCALL(sched_yield),
 #endif
+#ifdef SYS_seccomp
+	SYSCALL(seccomp),
+#endif
 #ifdef SYS_security
 	SYSCALL(security),
 #endif
@@ -3870,6 +3999,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_statfs64
 	SYSCALL(statfs64),
 #endif
+#ifdef SYS_statx
+	SYSCALL(statx),
+#endif
 #ifdef SYS_stime
 	SYSCALL(stime),
 #endif
@@ -3987,6 +4119,9 @@ syscall_t syscalls[] = {
 #ifdef SYS_uselib
 	SYSCALL(uselib),
 #endif
+#ifdef SYS_userfaultfd
+	SYSCALL(userfaultfd),
+#endif
 #ifdef SYS_ustat
 	SYSCALL(ustat),
 #endif
